package com.mcwl.communityCenter.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.mcwl.common.constant.HttpStatus;
import com.mcwl.common.core.domain.R;
import com.mcwl.common.core.domain.entity.SysUser;
import com.mcwl.common.core.page.TableDataInfo;
import com.mcwl.common.utils.SecurityUtils;
import com.mcwl.common.utils.StringUtils;
import com.mcwl.communityCenter.domain.Community;
import com.mcwl.communityCenter.domain.CommunityUser;
import com.mcwl.communityCenter.domain.dto.CommunityListPageRes;
import com.mcwl.communityCenter.domain.dto.CommunityRes;
import com.mcwl.communityCenter.domain.dto.JoinCommunityRes;
import com.mcwl.communityCenter.domain.vo.CommunityVo;
import com.mcwl.communityCenter.mapper.CommunityMapper;
import com.mcwl.communityCenter.mapper.CommunityUserMapper;
import com.mcwl.communityCenter.service.CommunityService;
import com.mcwl.system.service.ISysUserService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.ZoneId;
import java.time.temporal.ChronoUnit;
import java.util.*;

@Service
@RequiredArgsConstructor
public class CommunityServiceImpl extends ServiceImpl<CommunityMapper, Community> implements CommunityService {

    private final CommunityUserMapper communityUserMapper;

    private final ISysUserService sysUserService;

    @Override
    public TableDataInfo listByPage(CommunityListPageRes communityListPageRes) {

        // 查询社区加入人数，以map形式返回，key为社区id，value为加入人数
        Map<Long, Integer> communityJoinNumMap = baseMapper.selectCommunityJoinNum();

        Page<Community> page = new Page<>(communityListPageRes.getPageNum(), communityListPageRes.getPageSize());


        boolean isAsc = Objects.equals(communityListPageRes.getIsAsc(), "asc");

        if (StringUtils.isBlank(communityListPageRes.getOrderByColumn())) {
            communityListPageRes.setOrderByColumn("create_time");
        }
        OrderItem orderItem = new OrderItem(communityListPageRes.getOrderByColumn(), isAsc);
        page.addOrder(orderItem);

        baseMapper.selectPage(page, new LambdaQueryWrapper<Community>()
                .eq(communityListPageRes.getCommunityTagId() != null, Community::getCommunityTagId, communityListPageRes.getCommunityTagId()));

        List<Community> communityList = page.getRecords();
        List<CommunityVo> communityVoList = new ArrayList<>();

        LocalDate currentLocalDate = LocalDate.now(ZoneId.systemDefault());
        for (Community community : communityList) {
            CommunityVo communityVo = new CommunityVo();
            BeanUtil.copyProperties(community, communityVo);
            // 当前时间和创建时间差
            Date createTime = community.getCreateTime();
            LocalDate createLocalDate = createTime.toInstant()
                    .atZone(ZoneId.systemDefault())
                    .toLocalDate();
            long daysBetween = ChronoUnit.DAYS.between(createLocalDate, currentLocalDate);
            communityVo.setCreateDay(daysBetween);
            communityVo.setJoinNum(communityJoinNumMap.getOrDefault(community.getId(), 0));
            communityVoList.add(communityVo);
        }

        TableDataInfo tableDataInfo = new TableDataInfo();
        tableDataInfo.setRows(communityVoList);
        tableDataInfo.setTotal(page.getTotal());
        tableDataInfo.setCode(HttpStatus.SUCCESS);
        tableDataInfo.setMsg("查询成功");

        return tableDataInfo;
    }

    @Override
    public void addCommunity(CommunityRes communityRes) {
        Community community = new Community();
        BeanUtil.copyProperties(communityRes, community);
        baseMapper.insert(community);

    }

    /**
     * 加入社区
     *
     * @param joinCommunityRes 加入社区请求参数
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public R<Object> joinCommunity(JoinCommunityRes joinCommunityRes) {
        Long tenantId = joinCommunityRes.getTenantId();
        Long communityId = joinCommunityRes.getCommunityId();
        Community community = baseMapper.getByTenantIdAndCommunityId(tenantId, communityId);
        if (Objects.isNull(community)) {
            return R.fail("社区不存在，请重新加入");
        }

        CommunityUser communityUser = communityUserMapper.selectByTenantIdAndCommunityIdAndUserId(tenantId,
                communityId,
                SecurityUtils.getUserId());

        if (Objects.nonNull(communityUser)) {
            return R.fail("您已加入该社区,不能重复加入");
        }

        SysUser user = SecurityUtils.getLoginUser().getUser();
        Double wallet = user.getWallet();
        Double price = community.getPrice();
        if (wallet < price) {
            return R.fail("钱包余额不足，请充值");
        }
        // 扣费
        BigDecimal priceBigDecimal = new BigDecimal(price.toString());
        BigDecimal walletBigDecimal = new BigDecimal(wallet.toString());
        user.setWallet(walletBigDecimal.subtract(priceBigDecimal).doubleValue());

        sysUserService.updateUser(user);

        communityUserMapper.insert(new CommunityUser(null, tenantId, communityId, SecurityUtils.getUserId()));


        return R.ok("加入成功");
    }


}
